/*
 * dirbar.cc - directory bar widget routines
 *
 * This file is part of the source distribution of 
 * Jafm - an icon based X11 desktop program 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 */


#include <qpushbt.h>
#include <qpainter.h>
#include <qlined.h>
#include <qregexp.h>

#include "dirbar.hh"
#include "icon.hh"


DirectoryBar::DirectoryBar (const QDir &d, QWidget *parent=0)
  : QWidget (parent) 
{
  dir = d ;

  setMinimumSize (150, 40) ;
  blist = new QList<DButton> ;
  blist->setAutoDelete (true) ;

  filter = 0 ;
  
  // put an icon in the corner
  logo = new Icon (0, Icon::suggest_zic ("."), 5) ;
  //  logo->rescale (4) ;
  logo->move (0,0) ;
  logo->set_accepts_drag_drop (true) ;
  logo->set_selected (true) ;

  dirName = new QLineEdit (this) ;
  CHECK_PTR (dirName) ;
  dirName->setFont (QFont ("courier", 14)) ;
  dirName->setGeometry (40,10,width ()-45,30) ;
  dirName->setText (d.absPath()) ;

  blist->setAutoDelete (true) ;
  connect (dirName, SIGNAL (returnPressed ()),
	   this, SLOT (parse()) );
  reset () ;
}

void DirectoryBar::reset (const QDir &d)
{
  dir = d ; // up to caller to make sure d is good?
  dirName->setText (d.path()) ;
  reset () ;

}

void DirectoryBar::parse (const char *c)
{
  // called by directory buttons
  // we know that c is a good path (?)

  dir.setPath (c) ;
  dir.setNameFilter (0) ;
  dirName->setText (c) ;
  reset () ;
  emit (dirChanged (dir)) ;
}

void DirectoryBar::parse () 
 {
  /* called by the line edit widget */

  if (strlen (dirName->text()) == 0) 
    dirName->setText ("/") ;
  
  // make sure it exists
  QDir d (dirName->text()) ;
  if (d.exists()) {
      if (d.isReadable()) {
	reset (dirName->text() ) ; 
	emit (dirChanged(dir)) ;
      }
  } else {
    // try a filter
    QRegExp r ("[^/]*$") ;
    int m = r.match (dirName->text()) ;
    if (m>0) {
      QString s (dirName->text()) ;
      int nn = s.length() - m +1 ;
      filter = new char[nn];
      strcpy (filter, ((const char *)s)+m) ;
      d.setPath (s.remove (m, 1000)) ;
      d.setNameFilter (filter) ;
    }
  }
  dir = d ;
  reset() ;
  emit (dirChanged (dir)) ;
}

void DirectoryBar::up ()
{
  dir.cdUp () ;
  debug ("Up to %s", dir.path()) ;
  reset (dir.path()) ;
}


void DirectoryBar::reset ()
{
  /* this resets the directory buttons */
  
  const char *path = dirName->text() ;

  blist->clear () ;
  
  QFontMetrics fm = QFontMetrics (QFont ("courier", 14)) ;
  uint x = 40 ;
  uint i = 0 ;
  uint w, j ;
  for (j= 0; j < strlen(path) ; ++j) {
    
    if (path[j] != '/') 
      continue ;
    
    // characters from i to j form segment
    w = fm.width (path+i, j+1-i) ;
    if (j == 0) w+= 4 ; // this is poor and error prone
    
    //    path[j+1] = '\0' ;
    blist->append (new DButton (this, path, j+1)) ;
    blist->current()->setGeometry (x,0,w,10) ;
    connect (blist->current(), SIGNAL ( clicked(const char *) ),
	     this, SLOT (parse(const char *)) ) ;
    blist->current()->show() ;
	     
    i = j+1 ;
    x += w ;
  } 
  // final one reaches across length of widget
  blist->append (new DButton (this, path,j)) ;
  // blist->current()->setAutoResize (false) ;
  blist->current()->setGeometry (x,0,width() - w,10) ;
  blist->current()->show() ;
  saveW = w ;
}

void DirectoryBar::paintEvent (QPaintEvent *)
{
  QPainter p ;
  p.begin (this) ;
  //  p.setFont ( QFont ("courier", 14, QFont::Bold)) ;
  //  p.drawText ( 0,30, (const char *)(dir.absPath()) ) ;
  // draw the logo
  QPen pen (colorGroup().dark(), 2) ;
  p.setPen (pen) ;
  p.drawLine (1,1,1,38) ;
  p.drawLine (1,1,38,1) ;
  pen.setColor(colorGroup().light()) ;
  p.setPen (pen) ;
  p.drawLine (1,38,37,38) ;
  p.drawLine (38,1,38,38) ;
  logo->draw (p) ;
  p.end () ;
}

void DirectoryBar::resizeEvent (QResizeEvent *)
{
  blist->last()-> resize (width() - saveW, 10) ;
  dirName->resize (width(), 30) ;
}

void DirectoryBar::indicateDragDrop (const QPoint &)
{
  logo->set_drag_drop (true) ;
  repaint () ;
}
  
void DirectoryBar::drag_drop_happened (const char *c)
{
  reset (c) ;
  emit (dirChanged (dir)) ;
  logo->set_drag_drop (false) ;
  repaint () ;
}
